# abc/websocket.py
import asyncio
import websockets
import logging

logging.basicConfig(level=logging.INFO)

class WebSocketServer:
    def __init__(self, host="0.0.0.0", port=8765):
        self.host = host
        self.port = port
        self.clients = set()

    async def handler(self, websocket):
        logging.info(f"New connection from {websocket.remote_address}")
        self.clients.add(websocket)
        try:
            async for message in websocket:
                logging.info(f"Received: {message}")
                # Broadcast message to all clients except the sender
                for client in self.clients:
                    if client != websocket:
                        await client.send(message)
                # Send an acknowledgment back to the sender
                await websocket.send(f"Echo: {message}")
        except websockets.exceptions.ConnectionClosed:
            logging.warning("Client disconnected")
        finally:
            self.clients.remove(websocket)

    async def start(self):
        server = await websockets.serve(self.handler, self.host, self.port)
        logging.info(f"WebSocket Server started on {self.host}:{self.port}")
        await server.wait_closed()  # Keep the server running


class WebSocketClient:
    def __init__(self, uri):
        self.uri = uri

    async def send_message(self, message):
        try:
            async with websockets.connect(self.uri) as websocket:
                logging.info(f"Connected to server at {self.uri}")
                await websocket.send(message)
                logging.info(f"Sent: {message}")
                response = await websocket.recv()
                logging.info(f"Received from server: {response}")
        except Exception as e:
            logging.error(f"Error: {e}")
